/*
 *  2014-09-12: this file was modified by International Business Machines Corporation.
 *  Modifications Copyright 2014 IBM Corporation.
 */

/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package java.beans;

import java.util.Hashtable;

public class PropertyEditorManager {

    private static String[] path = { "org.apache.harmony.beans.editors" }; //$NON-NLS-1$

    private static final Hashtable<Class<?>, Class<?>> registeredEditors = new Hashtable<Class<?>, Class<?>>();

    static{
        // init default editors for primitive types
        registeredEditors.put(Long.TYPE, getDefaultEditorClass("LongEditor"));
        registeredEditors.put(Boolean.TYPE, getDefaultEditorClass("BooleanEditor"));
        registeredEditors.put(Short.TYPE, getDefaultEditorClass("ShortEditor"));
        registeredEditors.put(Float.TYPE, getDefaultEditorClass("FloatEditor"));
        registeredEditors.put(Double.TYPE, getDefaultEditorClass("DoubleEditor"));
        registeredEditors.put(Byte.TYPE, getDefaultEditorClass("ByteEditor"));
        registeredEditors.put(Integer.TYPE, getDefaultEditorClass("IntEditor"));

    }

    static Class<?> getDefaultEditorClass(String className) {
        for (String apath : path) {
            if (apath == null) {
                continue;
            }
            try {
                Class<?> clazz = Class.forName(apath + '.' + className);
                clazz.asSubclass(PropertyEditor.class);
                return clazz;
            } catch (Exception e) {
                // Ignored
            }
        }
        return null;
    }

    public PropertyEditorManager() {
        // default constructor, do nothing
    }

    public static void registerEditor(Class<?> targetType, Class<?> editorClass) {
        if (targetType == null) {
            throw new NullPointerException();
        }

        SecurityManager sm = System.getSecurityManager();
        if (sm != null) {
            sm.checkPropertiesAccess();
        }
        if (editorClass != null) {
            registeredEditors.put(targetType, editorClass);
        } else {
            registeredEditors.remove(targetType);
        }
    }

    public static synchronized PropertyEditor findEditor(Class<?> targetType) {
        if (targetType == null) {
            throw new NullPointerException();
        }

        Class<?> editorClass = registeredEditors.get(targetType);
        if (editorClass != null) {
            try {
                return (PropertyEditor) editorClass.newInstance();
            } catch (Exception e) {
                // Ignored
            }
        }

        String editorClassName = targetType.getName() + "Editor"; //$NON-NLS-1$
        try {
            return loadEditor(targetType, editorClassName);
        } catch (Exception e) {
            // Ignored
        }

        String shortEditorClassName = (targetType.isPrimitive() ? (editorClassName
                .substring(0, 1).toUpperCase() + editorClassName.substring(1))
                : editorClassName
                        .substring(editorClassName.lastIndexOf('.') + 1));
        for (String element : path) {
            if (element == null) {
                continue;
            }
            try {
                return loadEditor(targetType, element + '.'
                        + shortEditorClassName);
            } catch (Exception e) {
                // Ignored
            }
        }
        return null;
    }

    private static PropertyEditor loadEditor(Class<?> targetType,
            String className) throws ClassNotFoundException,
            IllegalAccessException, InstantiationException {
        ClassLoader loader = targetType.getClassLoader();
        if (loader == null) {
            loader = ClassLoader.getSystemClassLoader();
        }
        try {
            return (PropertyEditor) loader.loadClass(className).newInstance();
        } catch (ClassNotFoundException e) {
            // Ignored
        }

        return (PropertyEditor) Thread.currentThread().getContextClassLoader()
                .loadClass(className).newInstance();
    }

    public static void setEditorSearchPath(String[] apath) {
        SecurityManager sm = System.getSecurityManager();
        if (sm != null) {
            sm.checkPropertiesAccess();
        }
        if(apath == null){
            apath = new String[0];
        }
        synchronized(PropertyEditorManager.class){
            path = apath;
        }
    }

    public static synchronized String[] getEditorSearchPath() {
        return path.clone();
    }
}
