/*===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 
 * IBM SDK, Java(tm) Technology Edition, v6
 * (C) Copyright IBM Corp. 2013, 2014. All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *===========================================================================
 */
/*
 * %W% %E%
 *
 * Copyright (c) 2006, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package java.io;

import java.nio.channels.FileChannel;
import sun.nio.ch.FileChannelImpl;


/**
 * A file output stream is an output stream for writing data to a 
 * <code>File</code> or to a <code>FileDescriptor</code>. Whether or not
 * a file is available or may be created depends upon the underlying
 * platform.  Some platforms, in particular, allow a file to be opened
 * for writing by only one <tt>FileOutputStream</tt> (or other
 * file-writing object) at a time.  In such situations the constructors in
 * this class will fail if the file involved is already open.
 *
 * <p><code>FileOutputStream</code> is meant for writing streams of raw bytes
 * such as image data. For writing streams of characters, consider using
 * <code>FileWriter</code>.
 *
 * @author  Arthur van Hoff
 * @version %I%, %G%
 * @see     java.io.File
 * @see     java.io.FileDescriptor
 * @see     java.io.FileInputStream
 * @since   JDK1.0
 */
public
class FileOutputStream extends OutputStream
{
    /**
     * The system dependent file descriptor. The value is
     * 1 more than actual file descriptor. This means that
     * the default value 0 indicates that the file is not open.
     */
    private FileDescriptor fd;

    private FileChannel channel= null;

    private boolean append = false;

    private Object closeLock = new Object();                                    //IBM-perf_FileIO
    private volatile boolean closed = false;                                    //IBM-perf_FileIO
                                                                                //IBM-perf_FileIO
    private static final ThreadLocal<Boolean> runningFinalize =                 //IBM-perf_FileIO
        new ThreadLocal<Boolean>();                                             //IBM-perf_FileIO
                                                                                //IBM-perf_FileIO
    private static boolean isRunningFinalize() {                                //IBM-perf_FileIO
        Boolean val;                                                            //IBM-perf_FileIO
        if ((val = runningFinalize.get()) != null)                              //IBM-perf_FileIO
            return val.booleanValue();                                          //IBM-perf_FileIO
        return false;                                                           //IBM-perf_FileIO
    }                                                                           //IBM-perf_FileIO

    /**
     * Creates an output file stream to write to the file with the 
     * specified name. A new <code>FileDescriptor</code> object is 
     * created to represent this file connection.
     * <p>
     * First, if there is a security manager, its <code>checkWrite</code> 
     * method is called with <code>name</code> as its argument.
     * <p>
     * If the file exists but is a directory rather than a regular file, does
     * not exist but cannot be created, or cannot be opened for any other
     * reason then a <code>FileNotFoundException</code> is thrown.
     *
     * @param      name   the system-dependent filename
     * @exception  FileNotFoundException  if the file exists but is a directory
     *                   rather than a regular file, does not exist but cannot
     *                   be created, or cannot be opened for any other reason
     * @exception  SecurityException  if a security manager exists and its
     *               <code>checkWrite</code> method denies write access
     *               to the file.
     * @see        java.lang.SecurityManager#checkWrite(java.lang.String)
     */
    public FileOutputStream(String name) throws FileNotFoundException {
	this(name != null ? new File(name) : null, false);
    }

    /**
     * Creates an output file stream to write to the file with the specified
     * <code>name</code>.  If the second argument is <code>true</code>, then
     * bytes will be written to the end of the file rather than the beginning.
     * A new <code>FileDescriptor</code> object is created to represent this
     * file connection.
     * <p>
     * First, if there is a security manager, its <code>checkWrite</code> 
     * method is called with <code>name</code> as its argument.
     * <p>
     * If the file exists but is a directory rather than a regular file, does
     * not exist but cannot be created, or cannot be opened for any other
     * reason then a <code>FileNotFoundException</code> is thrown.
     * 
     * @param     name        the system-dependent file name
     * @param     append      if <code>true</code>, then bytes will be written
     *                   to the end of the file rather than the beginning
     * @exception  FileNotFoundException  if the file exists but is a directory
     *                   rather than a regular file, does not exist but cannot
     *                   be created, or cannot be opened for any other reason.
     * @exception  SecurityException  if a security manager exists and its
     *               <code>checkWrite</code> method denies write access
     *               to the file.
     * @see        java.lang.SecurityManager#checkWrite(java.lang.String)
     * @since     JDK1.1
     */
    public FileOutputStream(String name, boolean append)
        throws FileNotFoundException
    {
        this(name != null ? new File(name) : null, append);
    }

    /**
     * Creates a file output stream to write to the file represented by 
     * the specified <code>File</code> object. A new 
     * <code>FileDescriptor</code> object is created to represent this 
     * file connection.
     * <p>
     * First, if there is a security manager, its <code>checkWrite</code> 
     * method is called with the path represented by the <code>file</code> 
     * argument as its argument.
     * <p>
     * If the file exists but is a directory rather than a regular file, does
     * not exist but cannot be created, or cannot be opened for any other
     * reason then a <code>FileNotFoundException</code> is thrown.
     *
     * @param      file               the file to be opened for writing.
     * @exception  FileNotFoundException  if the file exists but is a directory
     *                   rather than a regular file, does not exist but cannot
     *                   be created, or cannot be opened for any other reason
     * @exception  SecurityException  if a security manager exists and its
     *               <code>checkWrite</code> method denies write access
     *               to the file.
     * @see        java.io.File#getPath()
     * @see        java.lang.SecurityException
     * @see        java.lang.SecurityManager#checkWrite(java.lang.String)
     */
    public FileOutputStream(File file) throws FileNotFoundException {
	this(file, false);
    }

    /**
     * Creates a file output stream to write to the file represented by 
     * the specified <code>File</code> object. If the second argument is
     * <code>true</code>, then bytes will be written to the end of the file
     * rather than the beginning. A new <code>FileDescriptor</code> object is
     * created to represent this file connection.
     * <p>
     * First, if there is a security manager, its <code>checkWrite</code> 
     * method is called with the path represented by the <code>file</code> 
     * argument as its argument.
     * <p>
     * If the file exists but is a directory rather than a regular file, does
     * not exist but cannot be created, or cannot be opened for any other
     * reason then a <code>FileNotFoundException</code> is thrown.
     *
     * @param      file               the file to be opened for writing.
     * @param     append      if <code>true</code>, then bytes will be written
     *                   to the end of the file rather than the beginning
     * @exception  FileNotFoundException  if the file exists but is a directory
     *                   rather than a regular file, does not exist but cannot
     *                   be created, or cannot be opened for any other reason
     * @exception  SecurityException  if a security manager exists and its
     *               <code>checkWrite</code> method denies write access
     *               to the file.
     * @see        java.io.File#getPath()
     * @see        java.lang.SecurityException
     * @see        java.lang.SecurityManager#checkWrite(java.lang.String)
     * @since 1.4
     */
    public FileOutputStream(File file, boolean append)
        throws FileNotFoundException
    {
        String name = (file != null ? file.getPath() : null);
	SecurityManager security = System.getSecurityManager();
	if (security != null) {
	    security.checkWrite(name);
	}
        if (name == null) {
            throw new NullPointerException();
        }
	fd = new FileDescriptor();
	fd.incrementAndGetUseCount();                                           //IBM-perf_FileIO
        this.append = append;
	if (append) {
	    openAppend(name);
	} else {
	    open(name);
	}
    }

    /**
     * Creates an output file stream to write to the specified file 
     * descriptor, which represents an existing connection to an actual 
     * file in the file system.
     * <p>
     * First, if there is a security manager, its <code>checkWrite</code> 
     * method is called with the file descriptor <code>fdObj</code> 
     * argument as its argument.
     *
     * @param      fdObj   the file descriptor to be opened for writing
     * @exception  SecurityException  if a security manager exists and its
     *               <code>checkWrite</code> method denies
     *               write access to the file descriptor
     * @see        java.lang.SecurityManager#checkWrite(java.io.FileDescriptor)
     */
    public FileOutputStream(FileDescriptor fdObj) {
	SecurityManager security = System.getSecurityManager();
	if (fdObj == null) {
	    throw new NullPointerException();
	}
	if (security != null) {
	    security.checkWrite(fdObj);
	}
	fd = fdObj;
	/*                                                                      //IBM-perf_FileIO
	 * FileDescriptor is being shared by streams.                           //IBM-perf_FileIO
	 * Ensure that it's GC'ed only when all the streams/channels are done   //IBM-perf_FileIO
	 * using it.                                                            //IBM-perf_FileIO
	 */                                                                     //IBM-perf_FileIO
	fd.incrementAndGetUseCount();                                           //IBM-perf_FileIO
    }

    /**
     * Opens a file, with the specified name, for writing.
     * @param name name of file to be opened
     */
    private native void open(String name) throws FileNotFoundException;

    /**
     * Opens a file, with the specified name, for appending.
     * @param name name of file to be opened
     */
    private native void openAppend(String name) throws FileNotFoundException;

    /**
     * Writes the specified byte to this file output stream. Implements 
     * the <code>write</code> method of <code>OutputStream</code>.
     *
     * @param      b   the byte to be written.
     * @exception  IOException  if an I/O error occurs.
     */
    public native void write(int b) throws IOException;

    /**
     * Writes a sub array as a sequence of bytes.
     * @param b the data to be written
     * @param off the start offset in the data
     * @param len the number of bytes that are written
     * @exception IOException If an I/O error has occurred.
     */
    private native void writeBytes(byte b[], int off, int len) throws IOException;

    /**
     * Writes <code>b.length</code> bytes from the specified byte array 
     * to this file output stream. 
     *
     * @param      b   the data.
     * @exception  IOException  if an I/O error occurs.
     */
    public void write(byte b[]) throws IOException {
	writeBytes(b, 0, b.length);
    }

    /**
     * Writes <code>len</code> bytes from the specified byte array 
     * starting at offset <code>off</code> to this file output stream. 
     *
     * @param      b     the data.
     * @param      off   the start offset in the data.
     * @param      len   the number of bytes to write.
     * @exception  IOException  if an I/O error occurs.
     */
    public void write(byte b[], int off, int len) throws IOException {
	writeBytes(b, off, len);
    }

    /**
     * Closes this file output stream and releases any system resources 
     * associated with this stream. This file output stream may no longer 
     * be used for writing bytes. 
     *
     * <p> If this stream has an associated channel then the channel is closed
     * as well.
     *
     * @exception  IOException  if an I/O error occurs.
     *
     * @revised 1.4
     * @spec JSR-51
     */
    public void close() throws IOException {
        synchronized (closeLock) {                                              //IBM-perf_FileIO
            if (closed) {                                                       //IBM-perf_FileIO
                return;                                                         //IBM-perf_FileIO
            }                                                                   //IBM-perf_FileIO
            closed = true;                                                      //IBM-perf_FileIO
        }                                                                       //IBM-perf_FileIO
                                                                                //IBM-perf_FileIO
        if (channel != null) {                                                  //IBM-perf_FileIO
            /*                                                                  //IBM-perf_FileIO
             * Decrement FD use count associated with the channel               //IBM-perf_FileIO
             * The use count is incremented whenever a new channel              //IBM-perf_FileIO
             * is obtained from this stream.                                    //IBM-perf_FileIO
             */                                                                 //IBM-perf_FileIO
            fd.decrementAndGetUseCount();                                       //IBM-perf_FileIO
            channel.close();
        }                                                                       //IBM-perf_FileIO
                                                                                //IBM-perf_FileIO
        /*                                                                      //IBM-perf_FileIO
         * Decrement FD use count associated with this stream                   //IBM-perf_FileIO
         */                                                                     //IBM-perf_FileIO
        int useCount = fd.decrementAndGetUseCount();                            //IBM-perf_FileIO
                                                                                //IBM-perf_FileIO
        /*                                                                      //IBM-perf_FileIO
         * If FileDescriptor is still in use by another stream, the finalizer   //IBM-perf_FileIO
         * will not close it.                                                   //IBM-perf_FileIO
         */                                                                     //IBM-perf_FileIO
        if ((useCount <= 0) || (!isRunningFinalize() && CloseFDWithStream)) {   //IBM-perf_FileIO
            close0();
        }
    }

    /**
     * Returns the file descriptor associated with this stream.
     *
     * @return  the <code>FileDescriptor</code> object that represents 
     *          the connection to the file in the file system being used 
     *          by this <code>FileOutputStream</code> object. 
     * 
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FileDescriptor
     */
     public final FileDescriptor getFD()  throws IOException {
	if (fd != null) return fd;
	throw new IOException();
     }
    
    /**
     * Returns the unique {@link java.nio.channels.FileChannel FileChannel}
     * object associated with this file output stream. </p>
     *
     * <p> The initial {@link java.nio.channels.FileChannel#position()
     * </code>position<code>} of the returned channel will be equal to the
     * number of bytes written to the file so far unless this stream is in
     * append mode, in which case it will be equal to the size of the file.
     * Writing bytes to this stream will increment the channel's position
     * accordingly.  Changing the channel's position, either explicitly or by
     * writing, will change this stream's file position.
     *
     * @return  the file channel associated with this file output stream
     *
     * @since 1.4
     * @spec JSR-51
     */
    public FileChannel getChannel() {
        synchronized (this) {                                                   //IBM-perf_FileIO
            if (channel == null) {                                              //IBM-perf_FileIO
                channel = FileChannelImpl.open(fd, false, true, this, append);  //IBM-perf_FileIO
                /*                                                              //IBM-perf_FileIO
                 * Increment fd's use count. Invoking the channel's close()     //IBM-perf_FileIO
                 * method will result in decrementing the use count set for     //IBM-perf_FileIO
                 * the channel.                                                 //IBM-perf_FileIO
                 */                                                             //IBM-perf_FileIO
                fd.incrementAndGetUseCount();                                   //IBM-perf_FileIO
			}                                                       //IBM-perf_FileIO
            return channel;                                                     //IBM-perf_FileIO
        }                                                                       //IBM-perf_FileIO
    }

    /**
     * Cleans up the connection to the file, and ensures that the 
     * <code>close</code> method of this file output stream is
     * called when there are no more references to this stream. 
     *
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FileInputStream#close()
     */
    protected void finalize() throws IOException {
        if (fd != null) {
            if (fd == FileDescriptor.out || fd == FileDescriptor.err) {         //IBM-perf_FileIO
                flush();
            } else {
                                                                                //IBM-perf_FileIO
                /*                                                              //IBM-perf_FileIO
                 * Finalizer should not release the FileDescriptor if another   //IBM-perf_FileIO
                 * stream is still using it. If the user directly invokes       //IBM-perf_FileIO
                 * close() then the FileDescriptor is also released.            //IBM-perf_FileIO
                 */                                                             //IBM-perf_FileIO
                runningFinalize.set(Boolean.TRUE);                              //IBM-perf_FileIO
                try {                                                           //IBM-perf_FileIO
                    close();                                                    //IBM-perf_FileIO
                } finally {                                                     //IBM-perf_FileIO
                    runningFinalize.set(Boolean.FALSE);                         //IBM-perf_FileIO
                }                                                               //IBM-perf_FileIO
            }
        }
    }

    private native void close0() throws IOException;

    private static native void initIDs();
                                                                                //IBM-perf_FileIO
    private static boolean CloseFDWithStream = true;                            //IBM-perf_FileIO
    
    static {
	initIDs();
	CloseFDWithStream = !((String)java.security.AccessController.doPrivileged( //IBM-perf_FileIO
	    new sun.security.action.GetPropertyAction(                          //IBM-perf_FileIO
	        "com.ibm.streams.CloseFDWithStream", "true"))).equalsIgnoreCase("false"); //IBM-perf_FileIO
    }
    /*iSeries specific change */                                                //IBM-iseries_bringup
    private static native void isIseries(int iSeriesInd);                       //IBM-iseries_bringup
                                                                                //IBM-iseries_bringup
    static {                                                                    //IBM-iseries_bringup
        if ( System.getProperty("os.name").compareTo("OS/400") == 0 ) {         //IBM-iseries_bringup
            isIseries(1);                                                       //IBM-iseries_bringup
        }                                                                       //IBM-iseries_bringup
    }                                                                           //IBM-iseries_bringup
    /*iSeries specific change */                                                //IBM-iseries_bringup

}
//IBM-iseries_bringup
//IBM-net_sun_giveback
//IBM-perf_FileIO
