package java.lang.ref;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 1998, 2014  All Rights Reserved
 */

/**
 * Abstract class which describes behavior common to all reference objects.
 *
 * @author		OTI
 * @version		initial
 * @since		1.2
 */
public abstract class Reference<T> extends Object {
	private T referent;

	private ReferenceQueue queue;
	private boolean enqueued;

/**
 * Make the referent null.  This does not force the reference object to be enqueued.
 */
public void clear() {
	referent = null;
}

/**
 * Force the reference object to be enqueued if it has been associated with a queue.
 *
 * @return	true if Reference is enqueued, false otherwise.
 */
public boolean enqueue() {
	return enqueueImpl();
}

/**
 * Return the referent of the reference object.
 *
 * @return	the referent to which reference refers,
 *			or null if object has been cleared.
 */
public T get() {
	return getImpl();
}

private native T getImpl();

/**
 * Return whether the reference object has been enqueued.
 *
 * @return	true if Reference has been enqueued, false otherwise.
 */
public boolean isEnqueued () {
	synchronized(this) {
		return enqueued;
	}
}

/**
 * Enqueue the reference object on the associated queue.
 *
 * @return	true if the Reference was successfully
 *			enqueued, false otherwise.
 */
boolean enqueueImpl() {
	ReferenceQueue tempQueue;
	boolean result;
	synchronized(this) {
		/* Static order for the following code (DO NOT CHANGE) */
		tempQueue = queue;
		queue = null;
		if(enqueued || tempQueue == null) return false;
		result = tempQueue.enqueue(this);
		if(result) {
			enqueued = true;
		}
		return result;
	}
}

/**
 * Constructs a new instance of this class.
 */
Reference() {
}

/**
 * Initialize a newly created reference object. Associate the
 * reference object with the referent.
 *
 * @param r the referent
 */
void initReference (Object r) {
	initReferenceImpl(r);
}

/**
 * Initialize a newly created reference object.  Associate the
 * reference object with the referent, and the specified ReferenceQueue.
 *
 * @param r the referent
 * @param q the ReferenceQueue
 */
void initReference (Object r, ReferenceQueue q) {
	queue = q;
	enqueued = false;
	initReferenceImpl(r);
}

/**
 * Associate the reference object with the referent.
 */
private final native void initReferenceImpl(Object r);

/**
 * Called when a Reference has been removed from its ReferenceQueue.
 * Set the enqueued field to false.
 */
void dequeue() {
	synchronized(this) {
		enqueued = false;
	}
}
}
