/*===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 
 * IBM SDK, Java(tm) Technology Edition, v6
 * (C) Copyright IBM Corp. 2013, 2014. All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *===========================================================================
 */
/*
 * %W% %E%
 *
 * Copyright (c) 2006, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package java.nio;

import java.io.FileDescriptor;                               //JSE-5702         //IBM-nio_vad
import sun.security.action.GetBooleanAction;                 //JSE-5702         //IBM-nio_vad
import sun.security.action.GetPropertyAction;                //JSE-5702         //IBM-nio_vad
import java.security.AccessController;                       //JSE-5702         //IBM-nio_vad

/**
 * A direct byte buffer whose content is a memory-mapped region of a file.
 *
 * <p> Mapped byte buffers are created via the {@link
 * java.nio.channels.FileChannel#map FileChannel.map} method.  This class
 * extends the {@link ByteBuffer} class with operations that are specific to
 * memory-mapped file regions.
 * 
 * <p> A mapped byte buffer and the file mapping that it represents remain
 * valid until the buffer itself is garbage-collected.
 *
 * <p> The content of a mapped byte buffer can change at any time, for example
 * if the content of the corresponding region of the mapped file is changed by
 * this program or another.  Whether or not such changes occur, and when they
 * occur, is operating-system dependent and therefore unspecified.
 * 
 * <a name="inaccess"><p> All or part of a mapped byte buffer may become
 * inaccessible at any time, for example if the mapped file is truncated.  An
 * attempt to access an inaccessible region of a mapped byte buffer will not
 * change the buffer's content and will cause an unspecified exception to be
 * thrown either at the time of the access or at some later time.  It is
 * therefore strongly recommended that appropriate precautions be taken to
 * avoid the manipulation of a mapped file by this program, or by a
 * concurrently running program, except to read or write the file's content.
 *
 * <p> Mapped byte buffers otherwise behave no differently than ordinary direct
 * byte buffers. </p>
 *
 *
 * @author Mark Reinhold
 * @author JSR-51 Expert Group
 * @version %I%, %E%
 * @since 1.4
 */

public abstract class MappedByteBuffer
    extends ByteBuffer
{

    // This is a little bit backwards: By rights MappedByteBuffer should be a
    // subclass of DirectByteBuffer, but to keep the spec clear and simple, and
    // for optimization purposes, it's easier to do it the other way around.
    // This works because DirectByteBuffer is a package-private class.

    // Volatile to make sure that the finalization thread sees the current
    // value of this so that a region is not accidentally unmapped again later.
    volatile boolean isAMappedBuffer;			// package-private

    //The variable 'fdForForce' is used on windows platform and it              //IBM-nio_vad
    //holds the value of fileDescriptor. This variable value is passed to windows  //IBM-nio_vad
    //API FlushFileBuffers in force0 method of MappedByteBuffer.c.              //IBM-nio_vad
    private FileDescriptor fdForForce;                  //JSE-5702              //IBM-nio_vad
    // This should only be invoked by the DirectByteBuffer constructors
    //
    MappedByteBuffer(int mark, int pos, int lim, int cap, // package-private
		     boolean mapped)
    {
	super(mark, pos, lim, cap);
	isAMappedBuffer = mapped;
	this.fdForForce = null;                          //JSE-5702             //IBM-nio_vad
    }

    MappedByteBuffer(int mark, int pos, int lim, int cap) { // package-private
	super(mark, pos, lim, cap);
	isAMappedBuffer = false;
	this.fdForForce = null;                          //JSE-5702             //IBM-nio_vad
    }

    //This should only be invoked by the DirectByteBuffer constructors.      //JSE-5702 //IBM-nio_vad
    //The file descriptor has valid value only when                          //JSE-5702 //IBM-nio_vad
    //com.ibm.mappedByteBufferForce is set to true. Otherwise, fd is null    //JSE-5702 //IBM-nio_vad
                                                                                //IBM-nio_vad
    MappedByteBuffer(int mark, int pos, int lim, int cap, // package-private //JSE-5702 //IBM-nio_vad
                   boolean mapped, FileDescriptor fd)                        //JSE-5702 //IBM-nio_vad
    {                                                                        //JSE-5702 //IBM-nio_vad
        super(mark, pos, lim, cap);                                          //JSE-5702 //IBM-nio_vad
        isAMappedBuffer = mapped;                                            //JSE-5702 //IBM-nio_vad
       //assigning the value of filedescriptor to fdForForce                 //JSE-5702 //IBM-nio_vad
        this.fdForForce = fd;                                                //JSE-5702 //IBM-nio_vad
    }                                                                        //JSE-5702 //IBM-nio_vad
    private void checkMapped() {
	if (!isAMappedBuffer)
	    // Can only happen if a luser explicitly casts a direct byte buffer
	    throw new UnsupportedOperationException();
    }

    private int pagePosition() {
        assert isAMappedBuffer;
        int ps = Bits.pageSize();
        int offset = (int)(address % ps);
        return (offset >= 0) ? offset : (ps + offset);
    }

    /**
     * Tells whether or not this buffer's content is resident in physical
     * memory.
     *
     * <p> A return value of <tt>true</tt> implies that it is highly likely
     * that all of the data in this buffer is resident in physical memory and
     * may therefore be accessed without incurring any virtual-memory page
     * faults or I/O operations.  A return value of <tt>false</tt> does not
     * necessarily imply that the buffer's content is not resident in physical
     * memory.
     *
     * <p> The returned value is a hint, rather than a guarantee, because the
     * underlying operating system may have paged out some of the buffer's data
     * by the time that an invocation of this method returns.  </p>
     *
     * @return  <tt>true</tt> if it is likely that this buffer's content
     *          is resident in physical memory
     */
    public final boolean isLoaded() {
	checkMapped();
        if ((address == 0) || (capacity() == 0))
            return true;
        int offset = pagePosition();
        return isLoaded0(address - offset, (long)capacity() + (long)offset, Bits.pageSize());
    }

    /**
     * Loads this buffer's content into physical memory.
     *
     * <p> This method makes a best effort to ensure that, when it returns,
     * this buffer's content is resident in physical memory.  Invoking this
     * method may cause some number of page faults and I/O operations to
     * occur. </p>
     *
     * @return  This buffer
     */
    public final MappedByteBuffer load() {
	checkMapped();
        if ((address == 0) || (capacity() == 0))
            return this;
        int offset = pagePosition();
        load0(address - offset, (long)capacity() + (long)offset, Bits.pageSize());
	return this;
    }

    /**
     * Forces any changes made to this buffer's content to be written to the
     * storage device containing the mapped file.
     *
     * <p> If the file mapped into this buffer resides on a local storage
     * device then when this method returns it is guaranteed that all changes
     * made to the buffer since it was created, or since this method was last
     * invoked, will have been written to that device.
     *
     * <p> If the file does not reside on a local device then no such guarantee
     * is made.
     *
     * <p> If this buffer was not mapped in read/write mode ({@link
     * java.nio.channels.FileChannel.MapMode#READ_WRITE}) then invoking this
     * method has no effect. </p>
     *
     * @return  This buffer
     */
    public final MappedByteBuffer force() {
	checkMapped();
        if ((address == 0) || (capacity() == 0))
            return this;
        int offset = pagePosition();
        force0(address - offset, (long)capacity() + (long)offset, fdForForce);  //JSE-5702 //IBM-nio_vad
	return this;
    }

    private native boolean isLoaded0(long address, long length, int pageSize);
    private native int load0(long address, long length, int pageSize);
    private native void force0(long address, long length, FileDescriptor fdForForce);  //JSE-5702 //IBM-nio_vad

}
//IBM-nio_vad
