/*
 * %W% %E%
 *
 * Copyright (c) 2006, 2009, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package java.util.jar;

import java.io.*;
import java.net.URL;
import java.lang.ref.SoftReference;
import java.util.*;
import java.util.zip.*;
import java.security.CodeSigner;
import java.security.CodeSource;
import java.security.cert.Certificate;
import java.security.AccessController;
import sun.misc.IOUtils;
import sun.security.action.GetPropertyAction;
import sun.security.util.ManifestEntryVerifier;
import sun.misc.SharedSecrets;
import com.ibm.jvm.Util;                        /*ibm@99278*/                   //IBM-wrt_bringup
import java.lang.reflect.Constructor;           /*ibm@99278*/                   //IBM-wrt_bringup
import java.lang.reflect.Field;                                                 //IBM-uti_zip_Oracle_giveback

/**
 * The <code>JarFile</code> class is used to read the contents of a jar file
 * from any file that can be opened with <code>java.io.RandomAccessFile</code>.
 * It extends the class <code>java.util.zip.ZipFile</code> with support
 * for reading an optional <code>Manifest</code> entry. The
 * <code>Manifest</code> can be used to specify meta-information about the
 * jar file and its entries.
 *
 * <p> Unless otherwise noted, passing a <tt>null</tt> argument to a constructor
 * or method in this class will cause a {@link NullPointerException} to be
 * thrown.
 *
 * @author  David Connelly
 * @version %I%, %G%
 * @see	    Manifest
 * @see     java.util.zip.ZipFile
 * @see     java.util.jar.JarEntry
 * @since   1.2
 */
public
class JarFile extends ZipFile {
    private SoftReference<Manifest> manRef;
    private Manifest manRT;                                /*ibm@99278*/        //IBM-wrt_bringup
    private JarEntry manEntry;
    private JarVerifier jv;
    private boolean jvInitialized;
    private boolean verify;
    private boolean computedHasClassPathAttribute;
    private boolean hasClassPathAttribute;

    // Set up JavaUtilJarAccess in SharedSecrets
    static {
        SharedSecrets.setJavaUtilJarAccess(new JavaUtilJarAccessImpl());
    }

    /**
     * The JAR manifest file name.
     */
    public static final String MANIFEST_NAME = "META-INF/MANIFEST.MF";

    /**
     * Creates a new <code>JarFile</code> to read from the specified
     * file <code>name</code>. The <code>JarFile</code> will be verified if
     * it is signed.
     * @param name the name of the jar file to be opened for reading
     * @throws IOException if an I/O error has occurred
     * @throws SecurityException if access to the file is denied
     *         by the SecurityManager
     */
    public JarFile(String name) throws IOException {
	this(new File(name), true, ZipFile.OPEN_READ);
    }

    /**
     * Creates a new <code>JarFile</code> to read from the specified
     * file <code>name</code>.
     * @param name the name of the jar file to be opened for reading
     * @param verify whether or not to verify the jar file if
     * it is signed.
     * @throws IOException if an I/O error has occurred
     * @throws SecurityException if access to the file is denied
     *         by the SecurityManager 
     */
    public JarFile(String name, boolean verify) throws IOException {
        this(new File(name), verify, ZipFile.OPEN_READ);
    }

    /**
     * Creates a new <code>JarFile</code> to read from the specified
     * <code>File</code> object. The <code>JarFile</code> will be verified if
     * it is signed.
     * @param file the jar file to be opened for reading
     * @throws IOException if an I/O error has occurred
     * @throws SecurityException if access to the file is denied
     *         by the SecurityManager
     */
    public JarFile(File file) throws IOException {
	this(file, true, ZipFile.OPEN_READ);
    }


    /**
     * Creates a new <code>JarFile</code> to read from the specified
     * <code>File</code> object.
     * @param file the jar file to be opened for reading
     * @param verify whether or not to verify the jar file if
     * it is signed.
     * @throws IOException if an I/O error has occurred
     * @throws SecurityException if access to the file is denied
     *         by the SecurityManager.
     */
    public JarFile(File file, boolean verify) throws IOException {
	this(file, verify, ZipFile.OPEN_READ);
    }


    /**
     * Creates a new <code>JarFile</code> to read from the specified
     * <code>File</code> object in the specified mode.  The mode argument
     * must be either <tt>OPEN_READ</tt> or <tt>OPEN_READ | OPEN_DELETE</tt>.
     *
     * @param file the jar file to be opened for reading
     * @param verify whether or not to verify the jar file if
     * it is signed.
     * @param mode the mode in which the file is to be opened
     * @throws IOException if an I/O error has occurred
     * @throws IllegalArgumentException
     *         if the <tt>mode</tt> argument is invalid
     * @throws SecurityException if access to the file is denied
     *         by the SecurityManager
     * @since 1.3
     */
    public JarFile(File file, boolean verify, int mode) throws IOException {
	super(file, mode);
	this.verify = verify;
    }

    /**
     * Returns the jar file manifest, or <code>null</code> if none.
     *
     * @return the jar file manifest, or <code>null</code> if none
     *
     * @throws IllegalStateException
     *         may be thrown if the jar file has been closed
     */
    public Manifest getManifest() throws IOException {
        return getManifestFromReference();                                      //IBM-wrt_bringup
    }

    private Manifest getManifestFromReference() throws IOException {
      if (Util.isRealTime()) {                                        /*ibm@99278*/ //IBM-wrt_bringup
          return getManifestRT();                                     /*ibm@99278*/ //IBM-wrt_bringup
      }                                                                         //IBM-wrt_bringup
	Manifest man = manRef != null ? manRef.get() : null;

	if (man == null) {
	  
	    JarEntry manEntry = getManEntry();
	    
	    // If found then load the manifest
	    if (manEntry != null) {
		if (verify) {
		    byte[] b = getBytes(manEntry);
		    man = new Manifest(new ByteArrayInputStream(b));
		    if (!jvInitialized) {
		        jv = new JarVerifier(b);
		    }
		} else {
		    man = new Manifest(super.getInputStream(manEntry));
		}
	        manRef = new SoftReference(man);
	    }
	}
	return man;
    }

    /*                                                                          //IBM-wrt_bringup
     * Created for ibm@99278 - should match getManifest method above.           //IBM-wrt_bringup
     */                                                                         //IBM-wrt_bringup
    private Manifest getManifestRT() throws IOException {                       //IBM-wrt_bringup
	if (manRT == null) {                                                    //IBM-wrt_bringup
            JarEntry manEntry = getManEntry();                                  //IBM-wrt_bringup
	    // If found then load the manifest                                  //IBM-wrt_bringup
	    if (manEntry != null) {                                             //IBM-wrt_bringup
		if (verify) {                                                   //IBM-wrt_bringup
		    byte[] b = getBytes(manEntry);                              //IBM-wrt_bringup
                    try {                                                       //IBM-wrt_bringup
                        Class[]  argsClass = new Class[] {InputStream.class};   //IBM-wrt_bringup
                        Object[] args      = new Object[] {new ByteArrayInputStream(b)}; //IBM-wrt_bringup
                        Constructor argsConstructor = Class.forName("java.util.jar.Manifest").getDeclaredConstructor(argsClass); //IBM-wrt_bringup
                        manRT = (Manifest) javax.realtime.ImmortalMemory.instance().newInstance(argsConstructor,args); //IBM-wrt_bringup
                    } catch ( Exception e ) {                                   //IBM-wrt_bringup
                        if (e instanceof java.lang.reflect.InvocationTargetException ) { //IBM-wrt_bringup
                             throw (IOException ) e.getCause();                 //IBM-wrt_bringup
                        } else {                                                //IBM-wrt_bringup
                             throw new IOException("Unable to create Manifest"); //IBM-wrt_bringup
                        }                                                       //IBM-wrt_bringup
                    }                                                           //IBM-wrt_bringup
		    if (!jvInitialized) {                                       //IBM-wrt_bringup
		        jv = new JarVerifier(b);                                //IBM-wrt_bringup
		    }                                                           //IBM-wrt_bringup
		} else {                                                        //IBM-wrt_bringup
                    try {                                                       //IBM-wrt_bringup
                        Class[]  argsClass = new Class[] {InputStream.class};   //IBM-wrt_bringup
                        Object[] args      = new Object[] {super.getInputStream(manEntry)}; //IBM-wrt_bringup
                        Constructor argsConstructor = Class.forName("java.util.jar.Manifest").getDeclaredConstructor(argsClass); //IBM-wrt_bringup
                        manRT = (Manifest) javax.realtime.ImmortalMemory.instance().newInstance(argsConstructor,args); //IBM-wrt_bringup
                    } catch ( Exception e ) {                                   //IBM-wrt_bringup
                        if (e instanceof java.lang.reflect.InvocationTargetException ) { //IBM-wrt_bringup
                             throw (IOException ) e.getCause();                 //IBM-wrt_bringup
                        } else {                                                //IBM-wrt_bringup
                             throw new IOException("Unable to create Manifest"); //IBM-wrt_bringup
                        }                                                       //IBM-wrt_bringup
                    }                                                           //IBM-wrt_bringup
		}                                                               //IBM-wrt_bringup
	    }                                                                   //IBM-wrt_bringup
	}                                                                       //IBM-wrt_bringup
	return manRT;                                                           //IBM-wrt_bringup
    }                                                                           //IBM-wrt_bringup
                                                                                //IBM-wrt_bringup
    private native String[] getMetaInfEntryNames();

    /**
     * Returns the <code>JarEntry</code> for the given entry name or
     * <code>null</code> if not found.
     *
     * @param name the jar file entry name
     * @return the <code>JarEntry</code> for the given entry name or
     *         <code>null</code> if not found.
     *
     * @throws IllegalStateException
     *         may be thrown if the jar file has been closed
     *
     * @see java.util.jar.JarEntry
     */
    public JarEntry getJarEntry(String name) {
	return (JarEntry)getEntry(name);
    }

    /**
     * Returns the <code>ZipEntry</code> for the given entry name or
     * <code>null</code> if not found.
     *
     * @param name the jar file entry name
     * @return the <code>ZipEntry</code> for the given entry name or
     *         <code>null</code> if not found
     *
     * @throws IllegalStateException
     *         may be thrown if the jar file has been closed
     *
     * @see java.util.zip.ZipEntry
     */
    public ZipEntry getEntry(String name) {
	ZipEntry ze = super.getEntry(name);
	if (ze != null) {
	    return new JarFileEntry(ze);
	}
	return null;
    }

    /**
     * Returns an enumeration of the zip file entries.
     */
    public Enumeration<JarEntry> entries() {
	final Enumeration enum_ = super.entries();
	return new Enumeration<JarEntry>() {
	    public boolean hasMoreElements() {
		return enum_.hasMoreElements();
	    }
	    public JarFileEntry nextElement() {
		ZipEntry ze = (ZipEntry)enum_.nextElement();
		return new JarFileEntry(ze);
	    }
	};
    }

    private class JarFileEntry extends JarEntry {
	JarFileEntry(ZipEntry ze) {
	    super(ze);
	}
	public Attributes getAttributes() throws IOException {
	    Manifest man = JarFile.this.getManifest();
	    if (man != null) {
		return man.getAttributes(getName());
	    } else {
		return null;
	    }
	}
	public Certificate[] getCertificates() {
            try {
                maybeInstantiateVerifier();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
	    if (certs == null && jv != null) {
		certs = jv.getCerts(JarFile.this, this);
	    }
	    return certs == null ? null : (Certificate[]) certs.clone();
	}
	public CodeSigner[] getCodeSigners() {
	    try {
		maybeInstantiateVerifier();
	    } catch (IOException e) {
		throw new RuntimeException(e);
	    }
	    if (signers == null && jv != null) {
		signers = jv.getCodeSigners(JarFile.this, this);
	    }
	    return signers == null ? null : (CodeSigner[]) signers.clone();
	}
    }
	    
    /*
     * Ensures that the JarVerifier has been created if one is
     * necessary (i.e., the jar appears to be signed.) This is done as
     * a quick check to avoid processing of the manifest for unsigned
     * jars.
     */
    private void maybeInstantiateVerifier() throws IOException {
        if (jv != null) {
            return;
        }

        if (verify) {
            String[] names = getMetaInfEntryNames();
            if (names != null) {
                for (int i = 0; i < names.length; i++) {
                    String name = names[i].toUpperCase(Locale.ENGLISH);
                    if (name.endsWith(".DSA") ||
                        name.endsWith(".RSA") ||
                        name.endsWith(".SF")) {
                        // Assume since we found a signature-related file
                        // that the jar is signed and that we therefore
                        // need a JarVerifier and Manifest
                        getManifest();
                        return;
                    }
                }
            }
            // No signature-related files; don't instantiate a
            // verifier
            verify = false;
        }
    }


    /*
     * Initializes the verifier object by reading all the manifest
     * entries and passing them to the verifier.
     */
    private void initializeVerifier() {
	ManifestEntryVerifier mev = null;

	// Verify "META-INF/" entries...
	try {
	    String[] names = getMetaInfEntryNames();
	    if (names != null) {
		for (int i = 0; i < names.length; i++) {
		    JarEntry e = getJarEntry(names[i]);
                    if (e == null) {
                        throw new JarException("corrupted jar file");
                    }
		    if (!e.isDirectory()) {
			if (mev == null) {
			    mev = new ManifestEntryVerifier
				(getManifestFromReference());
			}
			byte[] b = getBytes(e);
			if (b != null && b.length > 0) {
			    jv.beginEntry(e, mev);
			    jv.update(b.length, b, 0, b.length, mev);
			    jv.update(-1, null, 0, 0, mev);
			}
		    }
		}
	    }
	} catch (IOException ex) {
	    // if we had an error parsing any blocks, just
	    // treat the jar file as being unsigned
	    jv = null;
            verify = false;
            if (JarVerifier.debug != null) {
                JarVerifier.debug.println("jarfile parsing error!");
                ex.printStackTrace();
            }
        }

	// if after initializing the verifier we have nothing
	// signed, we null it out.

	if (jv != null) {

	    jv.doneWithMeta();
	    if (JarVerifier.debug != null) {
		JarVerifier.debug.println("done with meta!"); 
	    }

	    if (jv.nothingToVerify()) {
		if (JarVerifier.debug != null) {
		    JarVerifier.debug.println("nothing to verify!");
		}
		jv = null;
                verify = false;
	    }
	}
    }

    /*
     * Reads all the bytes for a given entry. Used to process the
     * META-INF files.
     */
    private byte[] getBytes(ZipEntry ze) throws IOException {
        InputStream is = super.getInputStream(ze);
        byte[] b = IOUtils.readFully(is, (int)ze.getSize(), true);
	is.close();
	return b;
    }

    /**
     * Returns an input stream for reading the contents of the specified
     * zip file entry.
     * @param ze the zip file entry
     * @return an input stream for reading the contents of the specified
     *         zip file entry
     * @throws ZipException if a zip file format error has occurred
     * @throws IOException if an I/O error has occurred
     * @throws SecurityException if any of the jar file entries
     *         are incorrectly signed.
     * @throws IllegalStateException
     *         may be thrown if the jar file has been closed
     */
    public synchronized InputStream getInputStream(ZipEntry ze) 
	throws IOException 
    {
        maybeInstantiateVerifier();
	if (jv == null) {
	    return super.getInputStream(ze);
	}
	if (!jvInitialized) {
	    initializeVerifier();
	    jvInitialized = true;
	    // could be set to null after a call to
	    // initializeVerifier if we have nothing to
	    // verify
	    if (jv == null)
		return super.getInputStream(ze);
	}

	// wrap a verifier stream around the real stream
	return new JarVerifier.VerifierStream(
	    getManifestFromReference(),                                         //IBM-wrt_bringup
	    ze instanceof JarFileEntry ?
	    (JarEntry) ze : getJarEntry(ze.getName()),
	    super.getInputStream(ze),
	    jv);
    }

    // Statics for hand-coded Boyer-Moore search in hasClassPathAttribute()
    // The bad character shift for "class-path"
    private static int[] lastOcc;
    // The good suffix shift for "class-path"
    private static int[] optoSft;
    // Initialize the shift arrays to search for "class-path"
    private static char[] src = {'c','l','a','s','s','-','p','a','t','h'};
    static {
	lastOcc = new int[128];
	optoSft = new int[10];
	lastOcc[(int)'c']=1;
	lastOcc[(int)'l']=2;
	lastOcc[(int)'s']=5;
	lastOcc[(int)'-']=6;
	lastOcc[(int)'p']=7;
	lastOcc[(int)'a']=8;
	lastOcc[(int)'t']=9;
	lastOcc[(int)'h']=10;
	for (int i=0; i<9; i++)
	    optoSft[i]=10;
	optoSft[9]=1;
    }
 
    private JarEntry getManEntry() {
	if (manEntry == null) {
	    // First look up manifest entry using standard name
	    manEntry = getJarEntry(MANIFEST_NAME);
            if (manEntry == null) {
                // If not found, then iterate through all the "META-INF/"
                // entries to find a match.
                String[] names = getMetaInfEntryNames();
                if (names != null) {
                    for (int i = 0; i < names.length; i++) {
                        if (MANIFEST_NAME.equals(
                                                 names[i].toUpperCase(Locale.ENGLISH))) {
                            manEntry = getJarEntry(names[i]);
                            break;
                        }
                    }
                }
            }
	}
	return manEntry;
    }

    // Returns true iff this jar file has a manifest with a class path
    // attribute. Returns false if there is no manifest or the manifest
    // does not contain a "Class-Path" attribute. Currently exported to
    // core libraries via sun.misc.SharedSecrets.
    boolean hasClassPathAttribute() throws IOException {
        if (computedHasClassPathAttribute) {
            return hasClassPathAttribute;
        }

        hasClassPathAttribute = false;
        if (!isKnownToNotHaveClassPathAttribute()) {
            JarEntry manEntry = getManEntry();
            if (manEntry != null) {
                                                                                //IBM-uti_zip_Oracle_giveback
                InputStream is = super.getInputStream(manEntry);                //IBM-uti_zip_Oracle_giveback
                                                                                //IBM-uti_zip_Oracle_giveback
                byte[] b = IOUtils.readFully(is, (int)manEntry.getSize(), true); //IBM-uti_zip_Oracle_giveback
                                                                                //IBM-uti_zip_Oracle_giveback
                int last = b.length - src.length;
                int i = 0;
                next:       
                while (i<=last) {
                    for (int j=9; j>=0; j--) {
                        char c = (char) b[i+j];
                        c = (((c-'A')|('Z'-c)) >= 0) ? (char)(c + 32) : c;
                        if (c != src[j]) {
                            i += Math.max(j + 1 - lastOcc[c&0x7F], optoSft[j]);
                            continue next;
                        }
                    }
                    hasClassPathAttribute = true;
                    break;
                }
                                                                                //IBM-uti_zip_Oracle_giveback
                /*                                                              //IBM-uti_zip_Oracle_giveback
                 * The input stream allocate a native inflater to read the streams. In order to //IBM-uti_zip_Oracle_giveback
                 * avoid the allocation of native memory again and again for the inflaters, the //IBM-uti_zip_Oracle_giveback
                 * inflater is not released and instead add to an inflater pool of the JarFile. Hence, //IBM-uti_zip_Oracle_giveback
                 * the inflater is not released till the corresponding JarFile is garbage collected.  //IBM-uti_zip_Oracle_giveback
                 * This can result in Native memory leak if Inflaters are allocated for many //IBM-uti_zip_Oracle_giveback
                 * jars. Hence it is better to release the inflater while closing the stream //IBM-uti_zip_Oracle_giveback
                 * itself if the possibility of using the jar again is less.    //IBM-uti_zip_Oracle_giveback
                 *                                                              //IBM-uti_zip_Oracle_giveback
                 * Here the stream is read just to check whether ClassPathAttribute is set for the //IBM-uti_zip_Oracle_giveback
                 * the JarFile or not. If ClassPathAttribute is not set, then the Inflater can be  //IBM-uti_zip_Oracle_giveback
                 * released while closing the stream.                           //IBM-uti_zip_Oracle_giveback
                 *                                                              //IBM-uti_zip_Oracle_giveback
                 * The attribute of InputStream can be accessed only by defining a public method or   //IBM-uti_zip_Oracle_giveback
                 * attribute. Hence, the attribute 'closeInflater' has to be set through reflection.  //IBM-uti_zip_Oracle_giveback
                 */                                                             //IBM-uti_zip_Oracle_giveback
                if (!(hasClassPathAttribute)) {                                 //IBM-uti_zip_Oracle_giveback
                    Class<?> cls = is.getClass();                               //IBM-uti_zip_Oracle_giveback
                    try {                                                       //IBM-uti_zip_Oracle_giveback
                        final Field closeInflater = cls.getDeclaredField("closeInflater"); //IBM-uti_zip_Oracle_giveback
                                                                                //IBM-uti_zip_Oracle_giveback
                        java.security.AccessController.doPrivileged(new java.security.PrivilegedAction<Void>() {  //IBM-uti_zip_Oracle_giveback
                            public Void run() {                                          //IBM-uti_zip_Oracle_giveback
                                closeInflater.setAccessible(true);                                   //IBM-uti_zip_Oracle_giveback
                                return null;                                             //IBM-uti_zip_Oracle_giveback
                            }                                                            //IBM-uti_zip_Oracle_giveback
                        });                                                     //IBM-uti_zip_Oracle_giveback
                                                                                //IBM-uti_zip_Oracle_giveback
                        closeInflater.setBoolean(is,true);                      //IBM-uti_zip_Oracle_giveback
                    } catch (Exception ex) {                                    //IBM-uti_zip_Oracle_giveback
                       // Ignore any failures. It falls back to old behavior of not closing inflater. //IBM-uti_zip_Oracle_giveback
                    }                                                           //IBM-uti_zip_Oracle_giveback
                                                                                //IBM-uti_zip_Oracle_giveback
                }                                                               //IBM-uti_zip_Oracle_giveback
                                                                                //IBM-uti_zip_Oracle_giveback
                is.close();                                                     //IBM-uti_zip_Oracle_giveback
            }
        }
                                                                                //IBM-uti_zip_Oracle_giveback
        computedHasClassPathAttribute = true;
                                                                                //IBM-uti_zip_Oracle_giveback
        return hasClassPathAttribute;
    }

    private static String javaHome;
    private static String[] jarNames;
    private boolean isKnownToNotHaveClassPathAttribute() {
        // Optimize away even scanning of manifest for jar files we
        // deliver which don't have a class-path attribute. If one of
        // these jars is changed to include such an attribute this code
        // must be changed.
        if (javaHome == null) {
            javaHome = (String) AccessController.doPrivileged(
                new GetPropertyAction("java.home"));
        }
                                                                                //IBM-wrt_bringup
        String name = getName();                                                //IBM-wrt_bringup
        String localJavaHome = javaHome;                                        //IBM-wrt_bringup
        if (Util.isRealTime()) {                                                //IBM-wrt_bringup
            if (name.startsWith(localJavaHome)) {                               //IBM-wrt_bringup
                if (name.endsWith("rt.jar") ||                                  //IBM-wrt_bringup
                    name.endsWith("sunrsasign.jar") ||                          //IBM-wrt_bringup
                    name.endsWith("jsse.jar") ||                                //IBM-wrt_bringup
                    name.endsWith("jce.jar") ||                                 //IBM-wrt_bringup
                    name.endsWith("charsets.jar") ||                            //IBM-wrt_bringup
                    name.endsWith("dnsns.jar") ||                               //IBM-wrt_bringup
                    name.endsWith("ldapsec.jar") ||                             //IBM-wrt_bringup
                    name.endsWith("localedata.jar") ||                          //IBM-wrt_bringup
                    name.endsWith("sunjce_provider.jar") ||                     //IBM-wrt_bringup
                    name.endsWith("sunpkcs11.jar")) {                           //IBM-wrt_bringup
                    return true;                                                //IBM-wrt_bringup
                }                                                               //IBM-wrt_bringup
            }                                                                   //IBM-wrt_bringup
            return false;                                                       //IBM-wrt_bringup
        }                                                                       //IBM-wrt_bringup
                                                                                //IBM-wrt_bringup
                                                                                //IBM-wrt_bringup
        if (jarNames == null) {
            String[] names = new String[10];
            String fileSep = File.separator;
            int i = 0;
            names[i++] = fileSep + "rt.jar";
            names[i++] = fileSep + "sunrsasign.jar";
            names[i++] = fileSep + "jsse.jar";
            names[i++] = fileSep + "jce.jar";
            names[i++] = fileSep + "charsets.jar";
            names[i++] = fileSep + "dnsns.jar";
            names[i++] = fileSep + "ldapsec.jar";
            names[i++] = fileSep + "localedata.jar";
            names[i++] = fileSep + "sunjce_provider.jar";
            names[i++] = fileSep + "sunpkcs11.jar";
            jarNames = names;
        }

        if (name.startsWith(localJavaHome)) {
            String[] names = jarNames;
            for (int i = 0; i < names.length; i++) {
                if (name.endsWith(names[i])) {
                    return true;
                }
            }
        }
        return false;
    }

    private synchronized void ensureInitialization() {
	try {
	    maybeInstantiateVerifier();
	} catch (IOException e) {
	    throw new RuntimeException(e);
	}
	if (jv != null && !jvInitialized) {
	    initializeVerifier();
	    jvInitialized = true;
	}
    }

    JarEntry newEntry(ZipEntry ze) {
	return new JarFileEntry(ze);
    }

    Enumeration<String> entryNames(CodeSource[] cs) {
	ensureInitialization();
	if (jv != null) {
	    return jv.entryNames(this, cs);
	}
	
	/*
	 * JAR file has no signed content. Is there a non-signing
	 * code source?
	 */
	boolean includeUnsigned = false;
	for (int i=0; i < cs.length; i++) {
	    if (cs[i].getCodeSigners() == null) {
		includeUnsigned = true;
		break;
	    }
	}
	if (includeUnsigned) {
	    return unsignedEntryNames();
	} else {
	    return new Enumeration<String>() {
	        public boolean hasMoreElements() {
		    return false;
	        }
	        public String nextElement() {
		    throw new NoSuchElementException();
	        }
	    };
	}
    }
	    
    /**
     * Returns an enumeration of the zip file entries
     * excluding internal JAR mechanism entries and including
     * signed entries missing from the ZIP directory.
     */
    Enumeration<JarEntry> entries2() {
	ensureInitialization();
	if (jv != null) {
	    return jv.entries2(this, super.entries());
	}

	// screen out entries which are never signed
	final Enumeration enum_ = super.entries();
	return new Enumeration<JarEntry>() {
	    ZipEntry entry;
	    public boolean hasMoreElements() {
		if (entry != null) {
		    return true;
		}
		while (enum_.hasMoreElements()) {
		    ZipEntry ze = (ZipEntry) enum_.nextElement();
		    if (JarVerifier.isSigningRelated(ze.getName())) {
			continue;
		    }
		    entry = ze;
		    return true;
		}
		return false;
	    }
	    public JarFileEntry nextElement() {
                if (hasMoreElements()) {
                     ZipEntry ze = entry;
                     entry = null;
                     return new JarFileEntry(ze);
                 }
                 throw new NoSuchElementException();
	    }
	};
    }
	    
    CodeSource[] getCodeSources(URL url) {
	ensureInitialization();
	if (jv != null) {
	    return jv.getCodeSources(this, url);
	}

	/*
	 * JAR file has no signed content. Is there a non-signing
	 * code source?
	 */
	Enumeration unsigned = unsignedEntryNames();
	if (unsigned.hasMoreElements()) {
	    return new CodeSource[] { JarVerifier.getUnsignedCS(url) };
	} else {
	    return null;
	}
    }

    private Enumeration<String> unsignedEntryNames() {
	final Enumeration entries = entries();
	return new Enumeration<String>() {
	    String name;

	    /*
	     * Grab entries from ZIP directory but screen out
	     * metadata.
	     */
	    public boolean hasMoreElements() {
		if (name != null) {
		    return true;
		}
		while (entries.hasMoreElements()) {
		    String value;
		    ZipEntry e = (ZipEntry) entries.nextElement();
		    value = e.getName();
		    if (e.isDirectory() || JarVerifier.isSigningRelated(value)) {
			continue;
		    }
		    name = value;
		    return true;
		}
		return false;
	    }
	    public String nextElement() {
                if (hasMoreElements()) {
                     String value = name;
                     name = null;
                     return value;
                 }
                 throw new NoSuchElementException();
	    }
	};
    }

    CodeSource getCodeSource(URL url, String name) {
	ensureInitialization();
	if (jv != null) {
	    if (jv.eagerValidation) {
		CodeSource cs = null;
		JarEntry je = getJarEntry(name);
		if (je != null) {
	            cs = jv.getCodeSource(url, this, je);
		} else {
	            cs = jv.getCodeSource(url, name);
		}
		return cs;
	    } else {
	        return jv.getCodeSource(url, name);
	    }
	}

	return JarVerifier.getUnsignedCS(url);
    }

    void setEagerValidation(boolean eager) {
	try {
	    maybeInstantiateVerifier();
	} catch (IOException e) {
	    throw new RuntimeException(e);
	}
	if (jv != null) {
	    jv.setEagerValidation(eager);
	}
    }

    List getManifestDigests() {
	ensureInitialization();
	if (jv != null) {
	    return jv.getManifestDigests();
	}
	return new ArrayList();
    }
}
//IBM-wrt_bringup
//IBM-uti_zip_Oracle_giveback
