/*
 *  2014-09-12: this file was modified by International Business Machines Corporation.
 *  Modifications Copyright 2014 IBM Corporation.
 */

/* 
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package java.util.logging;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.harmony.kernel.vm.VM;
import org.apache.harmony.logging.internal.nls.Messages;

/**
 * <code>Level</code> objects are used to indicate the level of logging. There
 * are a set of predefined logging levels, each associated with an integer
 * value. Enabling a certain logging level also enables all logging levels with
 * larger values.
 * <p>
 * The predefined levels in ascending order are FINEST, FINER, FINE, CONFIG,
 * INFO, WARNING, SEVERE. There are two additional predefined levels, which are
 * ALL and OFF. ALL indicates logging all messages, and OFF indicates logging no
 * messages.
 * </p>
 */
public class Level implements Serializable {

    private static final long serialVersionUID = -8176160795706313070L;

    private static final List<Level> levels = new ArrayList<Level>(9);
    
    private static final String DEFAULT_RESOURCE_BUNDLE ="sun.util.logging.resources.logging"; //$NON-NLS-1$

    /**
     * The OFF level provides no logging messages.
     */
    public static final Level OFF = new Level("OFF", Integer.MAX_VALUE,DEFAULT_RESOURCE_BUNDLE); //$NON-NLS-1$

    /**
     * The SEVERE level indicates a severe failure.
     */
    public static final Level SEVERE = new Level("SEVERE", 1000, DEFAULT_RESOURCE_BUNDLE); //$NON-NLS-1$

    /**
     * The WARNING level indicates a warning.
     */
    public static final Level WARNING = new Level("WARNING", 900, DEFAULT_RESOURCE_BUNDLE); //$NON-NLS-1$

    /**
     * The INFO level indicates an informative message.
     */
    public static final Level INFO = new Level("INFO", 800, DEFAULT_RESOURCE_BUNDLE); //$NON-NLS-1$

    /**
     * The CONFIG level indicates a static configuration message.
     */
    public static final Level CONFIG = new Level("CONFIG", 700, DEFAULT_RESOURCE_BUNDLE); //$NON-NLS-1$

    /**
     * The FINE level provides tracing messages.
     */
    public static final Level FINE = new Level("FINE", 500, DEFAULT_RESOURCE_BUNDLE); //$NON-NLS-1$

    /**
     * The FINER level provides more detailed tracing messages.
     */
    public static final Level FINER = new Level("FINER", 400, DEFAULT_RESOURCE_BUNDLE); //$NON-NLS-1$

    /**
     * The FINEST level provides highly detailed tracing messages.
     */
    public static final Level FINEST = new Level("FINEST", 300, DEFAULT_RESOURCE_BUNDLE); //$NON-NLS-1$

    /**
     * The ALL level provides all logging messages.
     */
    public static final Level ALL = new Level("ALL", Integer.MIN_VALUE, DEFAULT_RESOURCE_BUNDLE); //$NON-NLS-1$

    private String localLevelName;

    /**
     * Parses a level name into a <code>Level</code> object.
     * 
     * @param name
     *            the name of the desired level, which cannot be null
     * @return a <code>Level</code> object with the specified name
     * @throws NullPointerException
     *             if <code>name</code> is <code>null</code>.
     * @throws IllegalArgumentException
     *             if <code>name</code> is not valid.
     */
    public static Level parse(String name) throws IllegalArgumentException {
        if (name == null) {
            // logging.1C=The 'name' parameter is null.
            throw new NullPointerException(Messages.getString("logging.1C")); //$NON-NLS-1$
        }

        KnownLevel level;

        // Look for a known Level with the given non-localized name.
        level = KnownLevel.getLevelByName(name);
        if (level != null) {
            return level.object;
        }
        
        try {
            int nameAsInt = Integer.parseInt(name);
            level = KnownLevel.getLevelByValue(nameAsInt);
            if (level == null) {
                // add new Level
                Level levelObject = new Level(name, nameAsInt);
                level = KnownLevel.getLevelByValue(nameAsInt);
            }
            return level.object;
        } catch (NumberFormatException ex) {
        }
        
        
        level = KnownLevel.findByLocalizedName(name);
        if (level != null) {
            return level.object;
        }
        // logging.1D=Cannot parse this name: {0}
        throw new IllegalArgumentException(Messages.getString(
            "logging.1D", name)); //$NON-NLS-1$
    }

    /**
     * The name of this Level.
     * 
     * @serial
     */
    private final String name;

    /**
     * The integer value indicating the level.
     * 
     * @serial
     */
    final int value;

    /**
     * The name of the resource bundle used to localize the level name.
     * 
     * @serial
     */
    private final String resourceBundleName;

    /**
     * The resource bundle associated with this level, used to localize the
     * level name.
     */
    private transient ResourceBundle rb;

    /**
     * Constructs an instance of <code>Level</code> taking the supplied name
     * and level value.
     * 
     * @param name
     *            name of the level
     * @param level
     *            an integer value indicating the level
     * @throws NullPointerException
     *             if <code>name</code> is <code>null</code>.
     */
    protected Level(String name, int level) {
        this(name, level, null);
    }

    /**
     * Constructs an instance of <code>Level</code> taking the supplied name
     * and level value.
     * 
     * @param name
     *            name of the level
     * @param level
     *            an integer value indicating the level
     * @param resourceBundleName
     *            the name of the resource bundle to use
     * @throws NullPointerException
     *             if <code>name</code> is <code>null</code>.
     */
    protected Level(String name, int level, String resourceBundleName) {
        if (name == null) {
            // logging.1C=The 'name' parameter is null.
            throw new NullPointerException(Messages.getString("logging.1C")); //$NON-NLS-1$
        }
        this.name = name;
        this.value = level;
        this.resourceBundleName = resourceBundleName;
        this.localLevelName = resourceBundleName == null ? name : null;
        synchronized (levels) {
            levels.add(this);
            KnownLevel.add(this);
        }
    }

    /**
     * Gets the name of this <code>Level</code>.
     * 
     * @return the name of this <code>Level</code>
     */
    public String getName() {
        return this.name;
    }

    /**
     * Gets the name of the resource bundle associated with this
     * <code>Level</code>.
     * 
     * @return the name of the resource bundle associated with this
     *         <code>Level</code>
     */
    public String getResourceBundleName() {
        return this.resourceBundleName;
    }

    /**
     * Gets the integer value indicating this <code>Level</code>.
     * 
     * @return the integer value indicating this <code>Level</code>
     */
    public final int intValue() {
        return this.value;
    }

    /**
     * Serialization helper method to maintain singletons and add any new
     * levels.
     * 
     * @return The resolved instance.
     */
    private Object readResolve() {
        synchronized (levels) {
            for (Level level : levels) {
                if (value != level.value) {
                    continue;
                }
                if (!name.equals(level.name)) {
                    continue;
                }
                if (resourceBundleName == level.resourceBundleName) {
                    return level;
                } else if (resourceBundleName != null
                        && resourceBundleName.equals(level.resourceBundleName)) {
                    return level;
                }
            }
            // This is a new value, so add it.
            levels.add(this);
            return this;
        }
    }

    /**
     * Serialization helper to setup transient resource bundle instance.
     * 
     * @param in
     *            The input stream to read the instance data from.
     * @throws IOException
     *             if an IO error occurs.
     * @throws ClassNotFoundException
     *             if a class is not found.
     */
    private void readObject(ObjectInputStream in) throws IOException,
            ClassNotFoundException {
        in.defaultReadObject();
        if (resourceBundleName != null) {
            try {
                rb = ResourceBundle.getBundle(resourceBundleName);
            } catch (MissingResourceException e) {
                rb = null;
            }
        }
    }

    /**
     * Gets the localized name of this level. The default locale is used. If no
     * resource bundle is associated with this <code>Level</code>, the
     * original level name is returned.
     * 
     * @return the localized name of this level
     */
    public String getLocalizedName() {
        return getLocalizedLevelName();
    }
    
    final String getLevelName() {
        return this.name;
    }

    final synchronized String getLocalizedLevelName() {
        if (localLevelName != null) {
            return localLevelName;
        }

        try {
            ResourceBundle rb = ResourceBundle.getBundle(resourceBundleName);
            localLevelName = rb.getString(name);
        } catch (Exception ex) {
            localLevelName = name;
        }
        return localLevelName;
    }

    /**
     * Compares two <code>Level</code> objects for equality. They are
     * considered to be equal if they have the same value.
     * 
     * @param o
     *            the other object to be compared with
     * @return <code>true</code> if this object equals to the supplied object,
     *         otherwise <code>false</code>
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }

        if (!(o instanceof Level)) {
            return false;
        }

        return ((Level) o).value == this.value;
    }

    /**
     * Returns the hash code of this <code>Level</code> object.
     * 
     * @return the hash code of this <code>Level</code> object
     */
    @Override
    public int hashCode() {
        return this.value;
    }

    /**
     * Returns the string representation of this <code>Level</code> object.
     * Usually this will include its name.
     * 
     * @return the string representation of this <code>Level</code> object
     */
    @Override
    public final String toString() {
        return this.name;
    }

    static Level findLevel(String name) {
        if (name == null) {
            throw new NullPointerException();
        }

        KnownLevel level;

        level = KnownLevel.getLevelByName(name);
        if (level != null) {
            return level.mirrorLevel;
        }

        try {
            int x = Integer.parseInt(name);
            level = KnownLevel.getLevelByValue(x);
            if (level == null) {
                level = KnownLevel.getLevelByValue(x);
            }
            return level.mirrorLevel;
        } catch (NumberFormatException ex) {
        }

        level = KnownLevel.findLevelByLocalLevelName(name);
        if (level != null) {
            return level.mirrorLevel;
        }

        return null;
    }

    static final class KnownLevel {
        private static Map<String, List<KnownLevel>> nameLevelMap =
                                    new HashMap<String, List<KnownLevel>>();
        private static Map<Integer, List<KnownLevel>> integerLevelMap =
                                    new HashMap<Integer, List<KnownLevel>>();
        final Level object; 
        final Level mirrorLevel;
        KnownLevel(Level l) {
            this.object = l;
            if (l.getClass() == Level.class) {
                this.mirrorLevel = l;
            } else {
                this.mirrorLevel = new Level(l.name, l.value, l.resourceBundleName);
            }
        }

        static synchronized void add(Level l) {
            KnownLevel o = new KnownLevel(l);
            List<KnownLevel> list = nameLevelMap.get(l.name);
            if (list == null) {
                list = new ArrayList<KnownLevel>();
                nameLevelMap.put(l.name, list);
            }
            list.add(o);

            list = integerLevelMap.get(l.value);
            if (list == null) {
                list = new ArrayList<KnownLevel>();
                integerLevelMap.put(l.value, list);
            }
            list.add(o);
        }

        static synchronized KnownLevel getLevelByName(String name) {
            List<KnownLevel> list = nameLevelMap.get(name);
            if (list != null) {
                return list.get(0);
            }
            return null;
        }

        static synchronized KnownLevel getLevelByValue(int value) {
            List<KnownLevel> list = integerLevelMap.get(value);
            if (list != null) {
                return list.get(0);
            }
            return null;
        }

        static synchronized KnownLevel findLevelByLocalLevelName(String name) {
            for (List<KnownLevel> levels : nameLevelMap.values()) {
                for (KnownLevel l : levels) {
                    String lname = l.object.getLocalizedLevelName();
                    if (name.equals(lname)) {
                        return l;
                    }
                }
            }
            return null;
        }

        static synchronized KnownLevel findByLocalizedName(String name) {
            for (List<KnownLevel> levels : nameLevelMap.values()) {
                for (KnownLevel l : levels) {
                    String lname = l.object.getLocalizedName();
                    if (name.equals(lname)) {
                        return l;
                    }
                }
            }
            return null;
        }

        static synchronized KnownLevel matches(Level l) {
            List<KnownLevel> list = nameLevelMap.get(l.name);
            if (list != null) {
                for (KnownLevel level : list) {
                    Level l2 = level.mirrorLevel;
                    if (l.value == l2.value &&
                           (l.resourceBundleName == l2.resourceBundleName ||
                               (l.resourceBundleName != null &&
                                l.resourceBundleName.equals(l2.resourceBundleName)))) {
                        return level;
                    }
                }
            }
            return null;
        }
    }
}
