/*===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 
 * IBM SDK, Java(tm) Technology Edition, v6
 * (C) Copyright IBM Corp. 2013, 2014. All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *===========================================================================
 */
/*
 * %W% %E%
 *
 * Copyright (c) 2006, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javax.management.remote.rmi;

import java.io.IOException;
import java.io.SerializablePermission;
import java.rmi.Remote;
import java.rmi.NoSuchObjectException;
import java.security.AccessControlContext;
import java.security.AccessController;
import java.security.Permissions;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.security.ProtectionDomain;
import java.util.Map;
import java.util.Collections;
import javax.rmi.PortableRemoteObject;
import javax.security.auth.Subject;

/**
 * <p>An {@link RMIServerImpl} that is exported through IIOP and that
 * creates client connections as RMI objects exported through IIOP.
 * User code does not usually reference this class directly.</p>
 *
 * @see RMIServerImpl
 *
 * @since 1.5
 * @since.unbundled 1.0
 */
public class RMIIIOPServerImpl extends RMIServerImpl {
// special ACC used to initialize the IIOP stub
// the only allowed privilege is SerializablePermission("enableSubclassImplementation")
    private static final AccessControlContext STUB_ACC;

    static {
	Permissions p = new Permissions();
	p.add(new SerializablePermission("enableSubclassImplementation"));
	STUB_ACC = new AccessControlContext(
	    new ProtectionDomain[]{
		new ProtectionDomain(null, p)
	    }
	);
    }
    /**
     * <p>Creates a new {@link RMIServerImpl}.</p>
     *
     * @param env the environment containing attributes for the new
     * <code>RMIServerImpl</code>.  Can be null, which is equivalent
     * to an empty Map.
     *
     * @exception IOException if the RMI object cannot be created.
     */
    public RMIIIOPServerImpl(Map<String,?> env)
	    throws IOException {
	super(env);

	this.env = (env == null) ? Collections.EMPTY_MAP : env;

        callerACC = AccessController.getContext();
    }

    protected void export() throws IOException {
	PortableRemoteObject.exportObject(this);
    }

    protected String getProtocol() {
	return "iiop";
    }

    /**
     * <p>Returns an IIOP stub.</p>
     * The stub might not yet be connected to the ORB. The stub will
     * be serializable only if it is connected to the ORB.
     * @return an IIOP stub.
     * @exception IOException if the stub cannot be created - e.g the
     *            RMIIIOPServerImpl has not been exported yet.
     **/
    public Remote toStub() throws IOException {
	if (System.getSecurityManager() == null) {
	    return PortableRemoteObject.toStub(this);
	} else {
	    try {
	        return AccessController.doPrivileged(new PrivilegedExceptionAction<Remote>() {
	
	            @Override
	            public Remote run() throws Exception {
	                return PortableRemoteObject.toStub(RMIIIOPServerImpl.this);
	            }
	        }, STUB_ACC);
	    } catch (PrivilegedActionException e) {
	        if (e.getException() instanceof NoSuchObjectException) {
	            throw (NoSuchObjectException)e.getException();
	        }
	        throw new RuntimeException("Unexpected exception type", e.getException());
	    }
	}
    }

    /**
     * <p>Creates a new client connection as an RMI object exported
     * through IIOP.
     *
     * @param connectionId the ID of the new connection.  Every
     * connection opened by this connector server will have a
     * different ID.  The behavior is unspecified if this parameter is
     * null.
     *
     * @param subject the authenticated subject.  Can be null.
     *
     * @return the newly-created <code>RMIConnection</code>.
     *
     * @exception IOException if the new client object cannot be
     * created or exported.
     */
    protected RMIConnection makeClient(String connectionId, Subject subject)
	    throws IOException {

	if (connectionId == null)
	    throw new NullPointerException("Null connectionId");

	RMIConnection client =
	    new RMIConnectionImpl(this, connectionId, getDefaultClassLoader(),
				  subject, env);
	PortableRemoteObject.exportObject(client);
	return client;
    }

    protected void closeClient(RMIConnection client) throws IOException {
	PortableRemoteObject.unexportObject(client);
    }
    
    /**
     * <p>Called by {@link #close()} to close the connector server by
     * unexporting this object.  After returning from this method, the
     * connector server must not accept any new connections.</p>
     *
     * @exception IOException if the attempt to close the connector
     * server failed.
     */
    protected void closeServer() throws IOException {
	PortableRemoteObject.unexportObject(this);
    }

    @Override
    RMIConnection doNewClient(final Object credentials) throws IOException {
        if (callerACC == null) {
            throw new SecurityException("AccessControlContext cannot be null");
        }
        try {
            return AccessController.doPrivileged(
                new PrivilegedExceptionAction<RMIConnection>() {
                    public RMIConnection run() throws IOException {
                        return superDoNewClient(credentials);
                    }
            }, callerACC);
        } catch (PrivilegedActionException pae) {
            throw (IOException) pae.getCause();
        }
    }

    RMIConnection superDoNewClient(Object credentials) throws IOException {
        return super.doNewClient(credentials);
    }

    private final Map env;
    private final AccessControlContext callerACC;
}
