/*===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 
 * IBM SDK, Java(tm) Technology Edition, v6
 * (C) Copyright IBM Corp. 2013, 2014. All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *===========================================================================
 */
/*
 * %W% %E%
 *
 * Copyright (c) 2006, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package javax.print.attribute.standard;

import javax.print.attribute.Attribute;
import javax.print.attribute.IntegerSyntax;
import javax.print.attribute.PrintServiceAttribute;

/**
 * Class PagesPerMinuteColor is an integer valued printing attribute that 
 * indicates the nominal number of pages per minute to the nearest whole number 
 * which may be generated by this printer when printing color (e.g., simplex, 
 * color). For purposes of this attribute, "color" means the same as for the 
 * {@link ColorSupported ColorSupported} attribute, namely, the device is 
 * capable of any type of color printing at all, including highlight color as 
 * well as full process color. This attribute is informative, not a service 
 * guarantee. Generally, it is the value used in the marketing literature to 
 * describe the color capabilities of this device. A value of 0 indicates a 
 * device that takes more than two minutes to process a page. If a color device 
 * has several color modes, it may use the pages-per- minute value for this 
 * attribute that corresponds to the mode that produces the highest number. 
 * <P>
 * A black and white only printer must not include the PagesPerMinuteColor 
 * attribute in its attribute set or service registration. If this attribute is 
 * present, then the {@link ColorSupported ColorSupported} printer description 
 * attribute must also be present and have a value of SUPPORTED. 
 * <P>
 * <B>IPP Compatibility:</B> The integer value gives the IPP integer value. The 
 * category name returned by <CODE>getName()</CODE> gives the IPP attribute 
 * name. 
 * <P>
 *
 * @author  Alan Kaminsky
 */
public final class PagesPerMinuteColor extends IntegerSyntax
	implements PrintServiceAttribute {

    static final long serialVersionUID = 1684993151687470944L;

    /**
     * Construct a new pages per minute color attribute with the given integer 
     * value. 
     *
     * @param  value  Integer value.
     *
     * @exception  IllegalArgumentException
     *    (Unchecked exception) Thrown if <CODE>value</CODE> is less than 0.
     */
    public PagesPerMinuteColor(int value) {
	super(value, 0, Integer.MAX_VALUE);
    }

    /**
     * Returns whether this pages per minute color attribute is equivalent to 
     * the passed in object. To be equivalent, all of the following conditions 
     * must be true: 
     * <OL TYPE=1>
     * <LI>
     * <CODE>object</CODE> is not null.
     * <LI>
     * <CODE>object</CODE> is an instance of class PagesPerMinuteColor.
     * <LI>
     * This pages per minute attribute's value and <CODE>object</CODE>'s 
     * value are equal. 
     * </OL>
     *
     * @param  object  Object to compare to.
     *
     * @return  True if <CODE>object</CODE> is equivalent to this pages per
     *          minute color attribute, false otherwise. 
     */
    public boolean equals(Object object) {
	return (super.equals(object) &&
		object instanceof PagesPerMinuteColor);
    }

    /**
     * Get the printing attribute class which is to be used as the "category" 
     * for this printing attribute value.
     * <P>
     * For class PagesPerMinuteColor, the 
     * category is class PagesPerMinuteColor itself. 
     *
     * @return  Printing attribute class (category), an instance of class
     *          {@link java.lang.Class java.lang.Class}.
     */
    public final Class<? extends Attribute> getCategory() {
	return PagesPerMinuteColor.class;
    }

    /**
     * Get the name of the category of which this attribute value is an 
     * instance. 
     * <P>
     * For class PagesPerMinuteColor, the 
     * category name is <CODE>"pages-per-minute-color"</CODE>. 
     *
     * @return  Attribute category name.
     */
    public final String getName() {
	return "pages-per-minute-color";
    }

}
