/*
 *  2014-09-12: this file was modified by International Business Machines Corporation.
 *  Modifications Copyright 2014 IBM Corporation.
 */

/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package javax.sql.rowset.serial;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Vector;

import javax.sql.rowset.RowSetWarning;

import sun.reflect.Reflection;

public class SerialJavaObject implements Serializable, Cloneable {

    private static final long serialVersionUID = -1465795139032831023L;

    /**
     * obj and chain are defined in serialized form.
     */
    private Object obj;

    /**
     * A container for the warnings issued on this <code>SerialJavaObject</code>
     * object. When there are multiple warnings, each warning is chained to the
     * previous warning.
     */
    @SuppressWarnings("rawtypes")
    // Required by serialized form, change to package access as RI
    Vector chain;

    private transient Field[] fields;

    public SerialJavaObject(Object obj) throws SerialException {
        // if any static fields are found, an exception
        // should be thrown

        // get Class. Object instance should always be available
        Class c = obj.getClass();

        // determine if object implements Serializable
        boolean serializableImpl = obj instanceof java.io.Serializable;

        // can only determine public fields (obviously). If
        // any of these are static, this should invalidate
        // the action of attempting to persist these fields
        // in a serialized form

        boolean anyStaticFields = false;
        fields = c.getFields();

        for (int i = 0; i < fields.length; i++) {
            if (fields[i].getModifiers() == Modifier.STATIC) {
                anyStaticFields = true;
            }
        }
        try {
            if (!serializableImpl) {
                throw new RowSetWarning("Warning, the object passed to "
                        + "the constructor does not implement Serializable");
            }
        } catch (RowSetWarning w) {
            setWarning(w);
        }

        if (anyStaticFields) {
            throw new SerialException("Located static fields in "
                    + "object instance. Cannot serialize");
        }

        this.obj = obj;
    }

    public Field[] getFields() throws SerialException {
        if (fields != null) {
            Class c = this.obj.getClass();
            checkPackageAccess(c);
            return c.getFields();
        } else {
            throw new SerialException("SerialJavaObject does not contain"
                    + " a serialized object instance");
        }
    }

    public Object getObject() throws SerialException {
        return obj;
    }

    /**
     * Registers the given warning.
     */
    private void setWarning(RowSetWarning e) {
        if (chain == null) {
            chain = new Vector();
        }
        chain.add(e);
    }

    /*
     * Check if the caller is allowed to access the specified class's package.  If access is denied,
     * throw a SecurityException.
     *
     */
    private void checkPackageAccess(Class clz) {
        SecurityManager s = System.getSecurityManager();
        if (s != null) {
            if (sun.reflect.misc.ReflectUtil.needsPackageAccessCheck(
                    getCallerClassLoader(), clz.getClassLoader())) {
                String name = clz.getName();
                int i = name.lastIndexOf('.');
                if (i != -1) {
                    s.checkPackageAccess(name.substring(0, i));
                }
            }
        }
    }

    /* Internal method used to get the caller's caller class loader.
     * Caution is required if you attempt to make changes as this method assumes
     * the following stack frame count:
     * 0: Reflection
     * 1: getCallerClassLoader
     * 2: checkPackageAccess
     * 3: getFields
     * 4: caller of getFields
     */
    private static ClassLoader getCallerClassLoader() {
        Class cc = Reflection.getCallerClass(4);
        ClassLoader cl = (cc != null) ? cc.getClassLoader() : null;
        return cl;
    }

}
